module Schleuder
  class SchleuderLogger < Log4r::Logger
    # Instantiates a logger to be used outside of list-contexts and sets
    # outputters and level.
    def initialize
      # The name 'log4r' is special: it is considered as meta-logger by Log4r
      # and receives log_internal()-messages.
      super 'log4r'
      # The initial log_level is inherited by all outputters.
      @level = eval("Log4r::#{Schleuder.config.log_level.upcase}")
      if Schleuder.config.log_file == 'syslog'
        formatter = Log4r::PatternFormatter.new(:pattern => "%M")
        require 'log4r/outputter/syslogoutputter'
        add Log4r::SyslogOutputter.new('syslog',
                                       { :level => @level,
                                         :ident => 'schleuder',
                                         :facility => "LOG_MAIL",
                                         :formatter => formatter }
                                      )
      else
        pattern = "%d Schleuder %l\t%M"
        formatter = Log4r::PatternFormatter.new(:pattern => pattern)
        add Log4r::FileOutputter.new('file',
                                     { :level => @level,
                                       :filename => Schleuder.config.log_file,
                                       :formatter => formatter }
                                    )
      end
      add MetaEmailOutputter.new('email',
                                 {:to => Schleuder.config.superadminaddr,
                                  :immediate_at => 'ERROR, FATAL'})
    end
  end
end
